/*
 * Copyright (c) 2015 - 2025 Intel Corporation
 * SPDX-License-Identifier: BSD-3-Clause
 */


#include <string>

namespace geopm
{
    const std::string drm_sysfs_json(void)
    {
        static const std::string result = R"-(
{
    "attributes": {
        "HWMON::IN0_INPUT": {
            "attribute": "in0_input",
            "scalar": 1e-3,
            "description": "The current voltage",
            "doc_domain": "gpu",
            "aggregation": "average",
            "behavior": "variable",
            "format": "double",
            "units": "volts",
            "writeable": false,
            "alias": ""
        },
        "HWMON::POWER1_MAX": {
            "attribute": "power1_max",
            "scalar": 1e-6,
            "description": "Requested power limit, sustained on average over DRM::HWMON::POWER1_MAX_INTERVAL",
            "doc_domain": "gpu",
            "aggregation": "sum",
            "behavior": "variable",
            "format": "double",
            "units": "watts",
            "writeable": true,
            "alias": "GPU_POWER_LIMIT_CONTROL"
        },
        "HWMON::POWER1_RATED_MAX": {
            "attribute": "power1_rated_max",
            "scalar": 1e-6,
            "description": "Default thermal design power limit",
            "doc_domain": "gpu",
            "aggregation": "sum",
            "behavior": "variable",
            "format": "double",
            "units": "watts",
            "writeable": false,
            "alias": "GPU_POWER_LIMIT_DEFAULT"
        },
        "HWMON::POWER1_MAX_INTERVAL": {
            "attribute": "power1_max_interval",
            "scalar": 1e-3,
            "description": "Requested time window over which DRM::HWMON::POWER1_MAX is sustained on average",
            "doc_domain": "gpu",
            "aggregation": "expect_same",
            "behavior": "variable",
            "format": "double",
            "units": "seconds",
            "writeable": true,
            "alias": "GPU_POWER_TIME_WINDOW_CONTROL"
        },
        "HWMON::POWER1_CRIT": {
            "attribute": "power1_crit",
            "scalar": 1e-6,
            "description": "Critical power limit, influencing the power manager's throttling decisions",
            "doc_domain": "gpu",
            "aggregation": "sum",
            "behavior": "variable",
            "format": "double",
            "units": "watts",
            "writeable": true,
            "alias": ""
        },
        "HWMON::CURR1_CRIT": {
            "attribute": "curr1_crit",
            "scalar": 1e-3,
            "description": "Critical current limit, influencing the power manager's throttling decisions",
            "doc_domain": "gpu",
            "aggregation": "sum",
            "behavior": "variable",
            "format": "double",
            "units": "amperes",
            "writeable": true,
            "alias": ""
        },
        "HWMON::ENERGY1_INPUT::GPU": {
            "attribute": "energy1_input",
            "scalar": 1e-6,
            "description": "GPU card-level energy counter",
            "doc_domain": "gpu",
            "aggregation": "sum",
            "behavior": "monotone",
            "format": "double",
            "units": "joules",
            "writeable": false,
            "alias": "GPU_ENERGY"
        },
        "HWMON::ENERGY1_INPUT::GPU_CHIP": {
            "attribute": "energy1_input",
            "scalar": 1e-6,
            "description": "GPU tile-level energy counter",
            "doc_domain": "gpu_chip",
            "aggregation": "sum",
            "behavior": "monotone",
            "format": "double",
            "units": "joules",
            "writeable": false,
            "alias": "GPU_CHIP_ENERGY"
        },
        "RPS_MIN_FREQ": {
            "attribute": "rps_min_freq_mhz",
            "scalar": 1e6,
            "description": "User-configured requested power state (RPS) for minimum GPU frequency",
            "doc_domain": "gpu_chip",
            "aggregation": "average",
            "behavior": "variable",
            "format": "double",
            "units": "hertz",
            "writeable": true,
            "alias": "GPU_CORE_FREQUENCY_MIN_CONTROL"
        },
        "RPS_MIN_FREQ_DEFAULT": {
            "attribute": ".defaults/rps_min_freq_mhz",
            "scalar": 1e6,
            "description": "The platform's default setting for DRM::RPS_MIN_FREQ",
            "doc_domain": "gpu_chip",
            "aggregation": "average",
            "behavior": "variable",
            "format": "double",
            "units": "hertz",
            "writeable": false,
            "alias": ""
        },
        "RPS_MAX_FREQ": {
            "attribute": "rps_max_freq_mhz",
            "scalar": 1e6,
            "description": "User-configured requested power state (RPS) for maximum GPU frequency",
            "doc_domain": "gpu_chip",
            "aggregation": "average",
            "behavior": "variable",
            "format": "double",
            "units": "hertz",
            "writeable": true,
            "alias": "GPU_CORE_FREQUENCY_MAX_CONTROL"
        },
        "RPS_MAX_FREQ_DEFAULT": {
            "attribute": ".defaults/rps_max_freq_mhz",
            "scalar": 1e6,
            "description": "The platform's default setting for DRM::RPS_MAX_FREQ",
            "doc_domain": "gpu_chip",
            "aggregation": "average",
            "behavior": "variable",
            "format": "double",
            "units": "hertz",
            "writeable": false,
            "alias": "GPU_CORE_FREQUENCY_MAX_AVAIL"
        },
        "RPS_CUR_FREQ": {
            "attribute": "rps_cur_freq_mhz",
            "scalar": 1e6,
            "description": "Latest GPU frequency cached by the driver",
            "doc_domain": "gpu_chip",
            "aggregation": "average",
            "behavior": "variable",
            "format": "double",
            "units": "hertz",
            "writeable": false,
            "alias": "GPU_CORE_FREQUENCY_STATUS"
        },
        "RPS_ACT_FREQ": {
            "attribute": "rps_act_freq_mhz",
            "scalar": 1e6,
            "description": "Actual latest GPU frequency",
            "doc_domain": "gpu_chip",
            "aggregation": "average",
            "behavior": "variable",
            "format": "double",
            "units": "hertz",
            "writeable": false,
            "alias": ""
        },
        "RPS_BOOST_FREQ": {
            "attribute": "rps_boost_freq_mhz",
            "scalar": 1e6,
            "description": "GPU boost frequency",
            "doc_domain": "gpu_chip",
            "aggregation": "average",
            "behavior": "variable",
            "format": "double",
            "units": "hertz",
            "writeable": false,
            "alias": ""
        },
        "BASE_ACT_FREQ": {
            "attribute": "base_act_freq_mhz",
            "scalar": 1e6,
            "description": "Actual GPU base-die frequency selected by the power manager",
            "doc_domain": "gpu_chip",
            "aggregation": "average",
            "behavior": "variable",
            "format": "double",
            "units": "hertz",
            "writeable": false,
            "alias": ""
        },
        "MEDIA_ACT_FREQ": {
            "attribute": "media_act_freq_mhz",
            "scalar": 1e6,
            "description": "Actual GPU media frequency",
            "doc_domain": "gpu_chip",
            "aggregation": "average",
            "behavior": "variable",
            "format": "double",
            "units": "hertz",
            "writeable": false,
            "alias": ""
        },
        "BASE_FREQ_FACTOR_STEP": {
            "attribute": "base_freq_factor",
            "scalar": 1.0,
            "description": "GPU base-die frequency factor scaled by 1/DRM::BASE_FREQ_FACTOR_SCALE",
            "doc_domain": "gpu_chip",
            "aggregation": "average",
            "behavior": "variable",
            "format": "double",
            "units": "none",
            "writeable": true,
            "alias": ""
        },
        "BASE_FREQ_FACTOR_STEP_DEFAULT": {
            "attribute": ".defaults/base_freq_factor",
            "scalar": 1.0,
            "description": "The platform's default setting for DRM::BASE_FREQ_FACTOR_STEP",
            "doc_domain": "gpu_chip",
            "aggregation": "average",
            "behavior": "variable",
            "format": "double",
            "units": "none",
            "writeable": false,
            "alias": ""
        },
        "BASE_FREQ_FACTOR_SCALE": {
            "attribute": "base_freq_factor.scale",
            "scalar": 1.0,
            "description": "Scaling factor for DRM::BASE_FREQ_FACTOR_STEP",
            "doc_domain": "gpu_chip",
            "aggregation": "average",
            "behavior": "variable",
            "format": "double",
            "units": "none",
            "writeable": false,
            "alias": ""
        },
        "MEDIA_FREQ_FACTOR_STEP": {
            "attribute": "media_freq_factor",
            "scalar": 1.0,
            "description": "GPU media frequency factor scaled by 1/DRM::MEDIA_FREQ_FACTOR_SCALE",
            "doc_domain": "gpu_chip",
            "aggregation": "average",
            "behavior": "variable",
            "format": "double",
            "units": "none",
            "writeable": true,
            "alias": ""
        },
        "MEDIA_FREQ_FACTOR_STEP_DEFAULT": {
            "attribute": ".defaults/media_freq_factor",
            "scalar": 1.0,
            "description": "The platform's default setting for DRM::MEDIA_FREQ_FACTOR_STEP",
            "doc_domain": "gpu_chip",
            "aggregation": "average",
            "behavior": "variable",
            "format": "double",
            "units": "none",
            "writeable": false,
            "alias": ""
        },
        "MEDIA_FREQ_FACTOR_SCALE": {
            "attribute": "media_freq_factor.scale",
            "scalar": 1.0,
            "description": "Scaling factor for DRM::MEDIA_FREQ_FACTOR_STEP",
            "doc_domain": "gpu_chip",
            "aggregation": "average",
            "behavior": "variable",
            "format": "double",
            "units": "none",
            "writeable": false,
            "alias": ""
        },
        "RPS_RP0_FREQ": {
            "attribute": "rps_RP0_freq_mhz",
            "scalar": 1e6,
            "description": "Maximum non-overclocked GPU frequency",
            "doc_domain": "gpu_chip",
            "aggregation": "average",
            "behavior": "variable",
            "format": "double",
            "units": "hertz",
            "writeable": false,
            "alias": ""
        },
        "BASE_RP0_FREQ": {
            "attribute": "base_RP0_freq_mhz",
            "scalar": 1e6,
            "description": "GPU base-die RP0 frequency",
            "doc_domain": "gpu_chip",
            "aggregation": "average",
            "behavior": "variable",
            "format": "double",
            "units": "hertz",
            "writeable": false,
            "alias": ""
        },
        "MEDIA_RP0_FREQ": {
            "attribute": "media_RP0_freq_mhz",
            "scalar": 1e6,
            "description": "GPU media RP0 frequency",
            "doc_domain": "gpu_chip",
            "aggregation": "average",
            "behavior": "variable",
            "format": "double",
            "units": "hertz",
            "writeable": false,
            "alias": ""
        },
        "MEM_RP0_FREQ": {
            "attribute": "mem_RP0_freq_mhz",
            "scalar": 1e6,
            "description": "GPU memory-die RP0 frequency",
            "doc_domain": "gpu_chip",
            "aggregation": "average",
            "behavior": "variable",
            "format": "double",
            "units": "hertz",
            "writeable": false,
            "alias": ""
        },
        "RPS_RP1_FREQ": {
            "attribute": "rps_RP1_freq_mhz",
            "scalar": 1e6,
            "description": "GPU RP1 nominal frequency",
            "doc_domain": "gpu_chip",
            "aggregation": "average",
            "behavior": "variable",
            "format": "double",
            "units": "hertz",
            "writeable": false,
            "alias": ""
        },
        "BASE_RPN_FREQ": {
            "attribute": "base_RPn_freq_mhz",
            "scalar": 1e6,
            "description": "GPU base-die RPn frequency",
            "doc_domain": "gpu_chip",
            "aggregation": "average",
            "behavior": "variable",
            "format": "double",
            "units": "hertz",
            "writeable": false,
            "alias": ""
        },
        "RPS_RPN_FREQ": {
            "attribute": "rps_RPn_freq_mhz",
            "scalar": 1e6,
            "description": "GPU minimum requestable frequency",
            "doc_domain": "gpu_chip",
            "aggregation": "average",
            "behavior": "variable",
            "format": "double",
            "units": "hertz",
            "writeable": false,
            "alias": "GPU_CORE_FREQUENCY_MIN_AVAIL"
        },
        "MEDIA_RPN_FREQ": {
            "attribute": "media_RPn_freq_mhz",
            "scalar": 1e6,
            "description": "GPU media RPn frequency",
            "doc_domain": "gpu_chip",
            "aggregation": "average",
            "behavior": "variable",
            "format": "double",
            "units": "hertz",
            "writeable": false,
            "alias": ""
        },
        "MEM_RPN_FREQ": {
            "attribute": "mem_RPn_freq_mhz",
            "scalar": 1e6,
            "description": "GPU memory-die RPn frequency",
            "doc_domain": "gpu_chip",
            "aggregation": "average",
            "behavior": "variable",
            "format": "double",
            "units": "hertz",
            "writeable": false,
            "alias": ""
        },
        "PUNIT_REQ_FREQ": {
            "attribute": "punit_req_freq_mhz",
            "scalar": 1e6,
            "description": "PUnit requested GPU frequency",
            "doc_domain": "gpu_chip",
            "aggregation": "average",
            "behavior": "variable",
            "format": "double",
            "units": "hertz",
            "writeable": false,
            "alias": ""
        },
        "RAPL_PL1_FREQ": {
            "attribute": "rapl_PL1_freq_mhz",
            "scalar": 1e6,
            "description": "GPU RAPL PL1 frequency",
            "doc_domain": "gpu_chip",
            "aggregation": "average",
            "behavior": "variable",
            "format": "double",
            "units": "hertz",
            "writeable": false,
            "alias": ""
        },
        "RC6_ENABLE": {
            "attribute": "rc6_enable",
            "scalar": 1.0,
            "description": "Enable RC6",
            "doc_domain": "gpu_chip",
            "aggregation": "average",
            "behavior": "variable",
            "format": "double",
            "units": "none",
            "writeable": true,
            "alias": ""
        },
        "RC6_RESIDENCY": {
            "attribute": "rc6_residency_ms",
            "scalar": 1e-3,
            "description": "Time spent in RC6 state",
            "doc_domain": "gpu_chip",
            "aggregation": "average",
            "behavior": "variable",
            "format": "double",
            "units": "seconds",
            "writeable": false,
            "alias": ""
        },
        "THROTTLE_REASON_PL1": {
            "attribute": "throttle_reason_pl1",
            "scalar": 1.0,
            "description": "Whether the GPU is currently being throttled due to PL1 (average power)",
            "doc_domain": "gpu_chip",
            "aggregation": "average",
            "behavior": "variable",
            "format": "double",
            "units": "none",
            "writeable": false,
            "alias": ""
        },
        "THROTTLE_REASON_PL2": {
            "attribute": "throttle_reason_pl2",
            "scalar": 1.0,
            "description": "Whether the GPU is currently being throttled due to PL2 (burst power)",
            "doc_domain": "gpu_chip",
            "aggregation": "average",
            "behavior": "variable",
            "format": "double",
            "units": "none",
            "writeable": false,
            "alias": ""
        },
        "THROTTLE_REASON_PL4": {
            "attribute": "throttle_reason_pl4",
            "scalar": 1.0,
            "description": "Whether the GPU is currently being throttled due to PL4 (current)",
            "doc_domain": "gpu_chip",
            "aggregation": "average",
            "behavior": "variable",
            "format": "double",
            "units": "none",
            "writeable": false,
            "alias": ""
        },
        "THROTTLE_REASON_PROCHOT": {
            "attribute": "throttle_reason_prochot",
            "scalar": 1.0,
            "description": "Whether the GPU is currently being throttled due to prochot",
            "doc_domain": "gpu_chip",
            "aggregation": "average",
            "behavior": "variable",
            "format": "double",
            "units": "none",
            "writeable": false,
            "alias": ""
        },
        "THROTTLE_REASON_RATL": {
            "attribute": "throttle_reason_ratl",
            "scalar": 1.0,
            "description": "Whether the GPU is currently being throttled due to RATL",
            "doc_domain": "gpu_chip",
            "aggregation": "average",
            "behavior": "variable",
            "format": "double",
            "units": "none",
            "writeable": false,
            "alias": ""
        },
        "THROTTLE_REASON_STATUS": {
            "attribute": "throttle_reason_status",
            "scalar": 1.0,
            "description": "Whether the GPU is currently being throttled for any reason",
            "doc_domain": "gpu_chip",
            "aggregation": "average",
            "behavior": "variable",
            "format": "double",
            "units": "none",
            "writeable": false,
            "alias": ""
        },
        "THROTTLE_REASON_THERMAL": {
            "attribute": "throttle_reason_thermal",
            "scalar": 1.0,
            "description": "Whether the GPU is currently being throttled for thermal reasons",
            "doc_domain": "gpu_chip",
            "aggregation": "average",
            "behavior": "variable",
            "format": "double",
            "units": "none",
            "writeable": false,
            "alias": ""
        },
        "THROTTLE_REASON_VR_TDC": {
            "attribute": "throttle_reason_vr_tdc",
            "scalar": 1.0,
            "description": "Whether the GPU is currently being throttled for VR TDC reasons",
            "doc_domain": "gpu_chip",
            "aggregation": "average",
            "behavior": "variable",
            "format": "double",
            "units": "none",
            "writeable": false,
            "alias": ""
        },
        "THROTTLE_REASON_VR_THERMALERT": {
            "attribute": "throttle_reason_thermalert",
            "scalar": 1.0,
            "description": "Whether the GPU is currently being throttled for VR thermal alert reasons",
            "doc_domain": "gpu_chip",
            "aggregation": "average",
            "behavior": "variable",
            "format": "double",
            "units": "none",
            "writeable": false,
            "alias": ""
        },
        "ACT_FREQ": {
            "attribute": "freq0/act_freq",
            "scalar": 1e6,
            "description": "Actual GPU frequency",
            "doc_domain": "gpu_chip",
            "aggregation": "average",
            "behavior": "variable",
            "format": "double",
            "units": "hertz",
            "writeable": false,
            "alias": ""
        },
        "CUR_FREQ": {
            "attribute": "freq0/cur_freq",
            "scalar": 1e6,
            "description": "Current GPU frequency",
            "doc_domain": "gpu_chip",
            "aggregation": "average",
            "behavior": "variable",
            "format": "double",
            "units": "hertz",
            "writeable": false,
            "alias": "GPU_CORE_FREQUENCY_STATUS"
        },
        "MAX_FREQ": {
            "attribute": "freq0/max_freq",
            "scalar": 1e6,
            "description": "Maximum GPU frequency",
            "doc_domain": "gpu_chip",
            "aggregation": "average",
            "behavior": "variable",
            "format": "double",
            "units": "hertz",
            "writeable": true,
            "alias": "GPU_CORE_FREQUENCY_MAX_CONTROL"
        },
        "MIN_FREQ": {
            "attribute": "freq0/min_freq",
            "scalar": 1e6,
            "description": "Minimum GPU frequency",
            "doc_domain": "gpu_chip",
            "aggregation": "average",
            "behavior": "variable",
            "format": "double",
            "units": "hertz",
            "writeable": true,
            "alias": "GPU_CORE_FREQUENCY_MIN_CONTROL"
        },
        "RP0_FREQ": {
            "attribute": "freq0/rp0_freq",
            "scalar": 1e6,
            "description": "Maximum non-overclocked GPU frequency",
            "doc_domain": "gpu_chip",
            "aggregation": "average",
            "behavior": "variable",
            "format": "double",
            "units": "hertz",
            "writeable": false,
            "alias": "GPU_CORE_FREQUENCY_MAX_AVAIL"
        },
        "RPA_FREQ": {
            "attribute": "freq0/rpa_freq",
            "scalar": 1e6,
            "description": "Achievable GPU frequency",
            "doc_domain": "gpu_chip",
            "aggregation": "average",
            "behavior": "variable",
            "format": "double",
            "units": "hertz",
            "writeable": false,
            "alias": ""
        },
        "RPE_FREQ": {
            "attribute": "freq0/rpe_freq",
            "scalar": 1e6,
            "description": "Efficient GPU frequency",
            "doc_domain": "gpu_chip",
            "aggregation": "average",
            "behavior": "variable",
            "format": "double",
            "units": "hertz",
            "writeable": false,
            "alias": ""
        },
        "RPN_FREQ": {
            "attribute": "freq0/rpn_freq",
            "scalar": 1e6,
            "description": "Minimum requestable GPU frequency",
            "doc_domain": "gpu_chip",
            "aggregation": "average",
            "behavior": "variable",
            "format": "double",
            "units": "hertz",
            "writeable": false,
            "alias": "GPU_CORE_FREQUENCY_MIN_AVAIL"
        },
        "IDLE_RESIDENCY": {
            "attribute": "gtidle/idle_residency_ms",
            "scalar": 1e-3,
            "description": "Time spent in idle state",
            "doc_domain": "gpu_chip",
            "aggregation": "average",
            "behavior": "monotone",
            "format": "double",
            "units": "seconds",
            "writeable": false,
            "alias": ""
        },
        "HWMON::ENERGY2_INPUT": {
            "attribute": "energy2_input",
            "scalar": 1e-6,
            "description": "GPU package energy counter",
            "doc_domain": "gpu",
            "aggregation": "sum",
            "behavior": "monotone",
            "format": "double",
            "units": "joules",
            "writeable": false,
            "alias": "GPU_CORE_ENERGY"
        },
        "HWMON::POWER2_CRIT": {
            "attribute": "power2_crit",
            "scalar": 1e-6,
            "description": "Critical power limit, influencing the power manager's throttling decisions",
            "doc_domain": "gpu",
            "aggregation": "sum",
            "behavior": "variable",
            "format": "double",
            "units": "watts",
            "writeable": true,
            "alias": ""
        }
    }
}
        )-";
        return result;
    }
}
